@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -44,9 +44,9 @@ struct log_item_s
   gpg_error_t err;     /* The logged error code.  */
   int intvalue;        /* A logged integer value.  */
   char *string;        /* A malloced string or NULL.  */
-  ksba_cert_t cert;    /* A certifciate or NULL. */
-  int have_err:1;
-  int have_intvalue:1;
+  ksba_cert_t cert;    /* A certificate or NULL. */
+  unsigned int have_err:1;
+  unsigned int have_intvalue:1;
 };
 typedef struct log_item_s *log_item_t;
 
@@ -72,15 +72,15 @@ struct audit_ctx_s
 
 
 static void writeout_para (audit_ctx_t ctx,
-                           const char *format, ...) GPGRT_GCC_A_PRINTF(2,3);
+                           const char *format, ...) GPGRT_ATTR_PRINTF(2,3);
 static void writeout_li (audit_ctx_t ctx, const char *oktext,
-                         const char *format, ...) GPGRT_GCC_A_PRINTF(3,4);
+                         const char *format, ...) GPGRT_ATTR_PRINTF(3,4);
 static void writeout_rem (audit_ctx_t ctx,
-                          const char *format, ...) GPGRT_GCC_A_PRINTF(2,3);
+                          const char *format, ...) GPGRT_ATTR_PRINTF(2,3);
 
 
 /* Add NAME to the list of help tags.  NAME needs to be a const string
-   an this function merly stores this pointer.  */
+   an this function merely stores this pointer.  */
 static void
 add_helptag (audit_ctx_t ctx, const char *name)
 {
@@ -164,7 +164,7 @@ audit_release (audit_ctx_t ctx)
 
 
 /* Set the type for the audit operation.  If CTX is NULL, this is a
-   dummy fucntion.  */
+   dummy function.  */
 void
 audit_set_type (audit_ctx_t ctx, audit_type_t type)
 {
@@ -716,7 +716,7 @@ list_cert (audit_ctx_t ctx, ksba_cert_t cert, int with_subj)
 
 
 /* List the chain of certificates from STARTITEM up to STOPEVENT.  The
-   certifcates are written out as comments.  */
+   certificates are written out as comments.  */
 static void
 list_certchain (audit_ctx_t ctx, log_item_t startitem, audit_event_t stopevent)
 {
@@ -1105,9 +1105,11 @@ proc_type_verify (audit_ctx_t ctx)
           switch (gpg_err_code (item->err))
             {
             case 0:                    ok = "good"; break;
+            case GPG_ERR_TRUE:         ok = "n/a";  break;
             case GPG_ERR_CERT_REVOKED: ok = "bad"; break;
             case GPG_ERR_NOT_ENABLED:  ok = "disabled"; break;
             case GPG_ERR_NO_CRL_KNOWN:
+            case GPG_ERR_INV_CRL_OBJ:
               ok = _("no CRL found for certificate");
               break;
             case GPG_ERR_CRL_TOO_OLD:
