@@ -27,9 +27,6 @@
 #  include <winsock2.h>
 # endif
 # include <windows.h>
-# ifndef VER_PLATFORM_WIN32_WINDOWS
-#  define VER_PLATFORM_WIN32_WINDOWS 1
-# endif
 #endif
 
 #include "gpg.h"
@@ -95,8 +92,15 @@ w32_system (const char *command)
           return -1;
         }
       if (DBG_EXTPROG)
-        log_debug ("ShellExecuteEx succeeded (hProcess=%p,hInstApp=%d)\n",
-                   see.hProcess, (int)see.hInstApp);
+        {
+          /* hInstApp has HINSTANCE type.  The documentations says
+             that it's not a true HINSTANCE and it can be cast only to
+             an int.  */
+          int hinstance = (intptr_t)see.hInstApp;
+
+          log_debug ("ShellExecuteEx succeeded (hProcess=%p,hInstApp=%d)\n",
+                     see.hProcess, hinstance);
+        }
 
       if (!see.hProcess)
         {
@@ -160,12 +164,11 @@ generate_photo_id (ctrl_t ctrl, PKT_public_key *pk,const char *photo_name)
 {
   PKT_user_id *uid;
   int error=1,i;
-  unsigned int len;
+  uint64_t len;
   char *filename;
   byte *photo=NULL;
   byte header[16];
   IOBUF file;
-  int overflow;
 
   header[0]=0x10; /* little side of photo header length */
   header[1]=0;    /* big side of photo header length */
@@ -233,11 +236,18 @@ generate_photo_id (ctrl_t ctrl, PKT_public_key *pk,const char *photo_name)
 	}
 
 
-      len=iobuf_get_filelength(file, &overflow);
-      if(len>6144 || overflow)
+      len = iobuf_get_filelength(file);
+      if(len>6144)
 	{
-	  tty_printf( _("This JPEG is really large (%d bytes) !\n"),len);
-	  if(!cpr_get_answer_is_yes("photoid.jpeg.size",
+          /* We silently skip JPEGs larger than 1MiB because we have a
+           * 2MiB limit on the user ID packets and we need some limit
+           * anyway because the returned u64 is larger than the u32 or
+           * OpenPGP.  Note that the diagnostic may print a wrong
+           * value if the value is really large; we don't fix this to
+           * avoid a string change.  */
+	  tty_printf( _("This JPEG is really large (%d bytes) !\n"), (int)len);
+	  if(len > 1024*1024
+             || !cpr_get_answer_is_yes("photoid.jpeg.size",
 			    _("Are you sure you want to use it? (y/N) ")))
 	  {
 	    iobuf_close(file);
@@ -375,16 +385,7 @@ static const char *
 get_default_photo_command(void)
 {
 #if defined(_WIN32)
-  OSVERSIONINFO osvi;
-
-  memset(&osvi,0,sizeof(osvi));
-  osvi.dwOSVersionInfoSize=sizeof(osvi);
-  GetVersionEx(&osvi);
-
-  if(osvi.dwPlatformId==VER_PLATFORM_WIN32_WINDOWS)
-    return "start /w %i";
-  else
-    return "!ShellExecute 400 %i";
+  return "!ShellExecute 400 %i";
 #elif defined(__APPLE__)
   /* OS X.  This really needs more than just __APPLE__. */
   return "open %I";
@@ -594,34 +595,35 @@ run_with_pipe (struct spawn_info *info, const void *image, u32 len)
                " external programs\n"));
   return;
 #else /* !EXEC_TEMPFILE_ONLY */
-  int to[2];
-  pid_t pid;
   gpg_error_t err;
   const char *argv[4];
-
-  err = gnupg_create_pipe (to);
-  if (err)
-    return;
+  gnupg_process_t proc;
 
   fill_command_argv (argv, info->command);
-  err = gnupg_spawn_process_fd (argv[0], argv+1, to[0], -1, -1, &pid);
-
-  close (to[0]);
-
+  err = gnupg_process_spawn (argv[0], argv+1, GNUPG_PROCESS_STDIN_PIPE,
+                             NULL, &proc);
   if (err)
-    {
-      log_error (_("unable to execute shell '%s': %s\n"),
-                 argv[0], gpg_strerror (err));
-      close (to[1]);
-    }
+    log_error (_("unable to execute shell '%s': %s\n"),
+               argv[0], gpg_strerror (err));
   else
     {
-      write (to[1], image, len);
-      close (to[1]);
+      int fd_in;
 
-      err = gnupg_wait_process (argv[0], pid, 1, NULL);
+      err = gnupg_process_get_fds (proc, 0, &fd_in, NULL, NULL);
+      if (err)
+        log_error ("unable to get pipe connection '%s': %s\n",
+                   argv[2], gpg_strerror (err));
+      else
+        {
+          write (fd_in, image, len);
+          close (fd_in);
+        }
+
+      err = gnupg_process_wait (proc, 1);
       if (err)
         log_error (_("unnatural exit of external program\n"));
+
+      gnupg_process_release (proc);
     }
 #endif /* !EXEC_TEMPFILE_ONLY */
 }
@@ -689,14 +691,11 @@ show_photo (const char *command, const char *name, const void *image, u32 len)
         log_error (_("system error while calling external program: %s\n"),
                    strerror (errno));
 #else
-      pid_t pid;
       gpg_error_t err;
       const char *argv[4];
 
       fill_command_argv (argv, spawn->command);
-      err = gnupg_spawn_process_fd (argv[0], argv+1, -1, -1, -1, &pid);
-      if (!err)
-        err = gnupg_wait_process (argv[0], pid, 1, NULL);
+      err = gnupg_process_spawn (argv[0], argv+1, 0, NULL, NULL);
       if (err)
         log_error (_("unnatural exit of external program\n"));
 #endif
