@@ -39,8 +39,52 @@
 #include "tofu.h"
 #include "key-clean.h"
 
+
+
+typedef struct key_item **KeyHashTable; /* see new_key_hash_table() */
+
+/*
+ * Structure to keep track of keys, this is used as an array where the
+ * item right after the last one has a keyblock set to NULL.  Maybe we
+ * can drop this thing and replace it by key_item
+ */
+struct key_array
+{
+  KBNODE keyblock;
+};
+
+
+/* Control information for the trust DB.  */
+static struct
+{
+  int init;
+  int level;
+  char *dbname;
+  int no_trustdb;
+} trustdb_args;
+
+
+/* Some globals.  */
+static struct key_item *utk_list;      /* all ultimately trusted keys */
+
+/* A list used to temporary store trusted keys and a flag indicated
+ * whether any --trusted-key option has been seen. */
+static struct key_item *trusted_key_list;
+static int any_trusted_key_seen;
+
+/* Flag whether a trustdb check is pending.  */
+static int pending_check_trustdb;
+
+
+
 static void write_record (ctrl_t ctrl, TRUSTREC *rec);
-static void do_sync(void);
+static void do_sync (void);
+static int validate_keys (ctrl_t ctrl, int interactive);
+
+
+/**********************************************
+ ************* some helpers *******************
+ **********************************************/
 
 
 
@@ -54,7 +98,7 @@ keyid_from_fpr20 (ctrl_t ctrl, const byte *fpr, u32 *keyid)
     keyid = dummy_keyid;
 
   /* Problem: We do only use fingerprints in the trustdb but
-   * we need the keyID here to indetify the key; we can only
+   * we need the keyID here to identify the key; we can only
    * use that ugly hack to distinguish between 16 and 20
    * bytes fpr - it does not work always so we better change
    * the whole validation code to only work with
@@ -68,7 +112,7 @@ keyid_from_fpr20 (ctrl_t ctrl, const byte *fpr, u32 *keyid)
       int rc;
 
       memset (&pk, 0, sizeof pk);
-      rc = get_pubkey_byfprint (ctrl, &pk, NULL, fpr, fprlen);
+      rc = get_pubkey_byfpr (ctrl, &pk, NULL, fpr, fprlen);
       if (rc)
         {
           log_printhex (fpr, fprlen,
@@ -88,40 +132,6 @@ keyid_from_fpr20 (ctrl_t ctrl, const byte *fpr, u32 *keyid)
   return keyid[1];
 }
 
-typedef struct key_item **KeyHashTable; /* see new_key_hash_table() */
-
-/*
- * Structure to keep track of keys, this is used as an array wherre
- * the item right after the last one has a keyblock set to NULL.
- * Maybe we can drop this thing and replace it by key_item
- */
-struct key_array
-{
-  KBNODE keyblock;
-};
-
-
-/* Control information for the trust DB.  */
-static struct
-{
-  int init;
-  int level;
-  char *dbname;
-  int no_trustdb;
-} trustdb_args;
-
-/* Some globals.  */
-static struct key_item *user_utk_list; /* temp. used to store --trusted-keys */
-static struct key_item *utk_list;      /* all ultimately trusted keys */
-
-static int pending_check_trustdb;
-
-static int validate_keys (ctrl_t ctrl, int interactive);
-
-
-/**********************************************
- ************* some helpers *******************
- **********************************************/
 
 static struct key_item *
 new_key_item (void)
@@ -245,11 +255,19 @@ tdb_register_trusted_keyid (u32 *keyid)
   k = new_key_item ();
   k->kid[0] = keyid[0];
   k->kid[1] = keyid[1];
-  k->next = user_utk_list;
-  user_utk_list = k;
+  k->next = trusted_key_list;
+  trusted_key_list = k;
 }
 
 
+/* This is called for the option --trusted-key to register these keys
+ * for later syncing them into the trustdb.  The special value "none"
+ * may be used to indicate that there is a trusted-key option but no
+ * key shall be inserted for it.  This "none" value is helpful to
+ * distinguish between changing the gpg.conf from a trusted-key to no
+ * trusted-key options at all.  Simply not specify the option would
+ * not allow to distinguish this case from the --no-options case as
+ * used for certain calls of gpg for example by gpg-wks-client.  */
 void
 tdb_register_trusted_key (const char *string)
 {
@@ -257,6 +275,9 @@ tdb_register_trusted_key (const char *string)
   KEYDB_SEARCH_DESC desc;
   u32 kid[2];
 
+  any_trusted_key_seen = 1;
+  if (!strcmp (string, "none"))
+    return;
   err = classify_user_id (string, &desc, 1);
   if (!err)
     {
@@ -378,11 +399,12 @@ verify_own_keys (ctrl_t ctrl)
           if (!add_utk (kid))
             log_info (_("key %s occurs more than once in the trustdb\n"),
                       keystr(kid));
-          else if ((rec.r.trust.flags & 1))
+          else if ((rec.r.trust.flags & 1)
+                   && any_trusted_key_seen)
             {
               /* Record marked as inserted via --trusted-key.  Is this
                * still the case?  */
-              for (k2 = user_utk_list; k2; k2 = k2->next)
+              for (k2 = trusted_key_list; k2; k2 = k2->next)
                 if (k2->kid[0] == kid[0] && k2->kid[1] == kid[1])
                   break;
               if (!k2) /* No - clear the flag.  */
@@ -406,7 +428,7 @@ verify_own_keys (ctrl_t ctrl)
     }
 
   /* Put any --trusted-key keys into the trustdb */
-  for (k = user_utk_list; k; k = k->next)
+  for (k = trusted_key_list; k; k = k->next)
     {
       if ( add_utk (k->kid) )
         { /* not yet in trustDB as ultimately trusted */
@@ -431,9 +453,9 @@ verify_own_keys (ctrl_t ctrl)
         }
     }
 
-  /* release the helper table table */
-  release_key_items (user_utk_list);
-  user_utk_list = NULL;
+  /* Release the helper table.  */
+  release_key_items (trusted_key_list);
+  trusted_key_list = NULL;
   return;
 }
 
@@ -730,7 +752,7 @@ tdb_check_or_update (ctrl_t ctrl)
       if (opt.interactive)
 	update_trustdb (ctrl);
       else if (!opt.no_auto_check_trustdb)
-	check_trustdb (ctrl);
+        check_trustdb (ctrl);
     }
 }
 
@@ -983,6 +1005,7 @@ update_min_ownertrust (ctrl_t ctrl, u32 *kid, unsigned int new_trust)
 
 /*
  * Clear the ownertrust and min_ownertrust values.
+ * Also schedule a revalidation if a stale validity record exists.
  *
  * Return: True if a change actually happened.
  */
@@ -1016,6 +1039,26 @@ tdb_clear_ownertrusts (ctrl_t ctrl, PKT_public_key *pk)
           do_sync ();
           return 1;
         }
+      else
+        {
+          /* Check whether we have a stale RECTYPE_VALID for that key
+           * and if its validity ist set, schedule a revalidation.  */
+          ulong recno = rec.r.trust.validlist;
+          while (recno)
+            {
+              read_record (recno, &rec, RECTYPE_VALID);
+              if (rec.r.valid.validity)
+                break;
+              recno = rec.r.valid.next;
+            }
+          if (recno)
+            {
+              if (DBG_TRUST)
+                log_debug ("stale validity value detected"
+                           " - scheduling check\n");
+              tdb_revalidation_mark (ctrl);
+            }
+        }
     }
   else if (gpg_err_code (err) != GPG_ERR_NOT_FOUND)
     {
@@ -1462,7 +1505,7 @@ list_trust_path( const char *username )
 /****************
  * Enumerate all keys, which are needed to build all trust paths for
  * the given key.  This function does not return the key itself or
- * the ultimate key (the last point in cerificate chain).  Only
+ * the ultimate key (the last point in certificate chain).  Only
  * certificate chains which ends up at an ultimately trusted key
  * are listed.	If ownertrust or validity is not NULL, the corresponding
  * value for the returned LID is also returned in these variable(s).
