@@ -93,7 +93,7 @@ struct server_local_s
    * multi_search_desc_len.  If a search description has ever been
    * allocated the allocated size is stored at multi_search_desc_size.
    * multi_search_store is allocated at the same size as
-   * multi_search_desc and used to provde backing store for the SN and
+   * multi_search_desc and used to provide backing store for the SN and
    * NAME elements of KEYBOX_SEARCH_DESC.  */
   KEYBOX_SEARCH_DESC search_desc;
   KEYBOX_SEARCH_DESC *multi_search_desc;
@@ -131,21 +131,25 @@ get_assuan_ctx_from_ctrl (ctrl_t ctrl)
 static gpg_error_t
 prepare_outstream (ctrl_t ctrl)
 {
-  int fd;
+  gnupg_fd_t fd;
+  estream_t out_fp = NULL;
 
   log_assert (ctrl && ctrl->server_local);
 
   if (ctrl->server_local->outstream)
     return 0;  /* Already enabled.  */
 
-  fd = translate_sys2libc_fd
-    (assuan_get_output_fd (get_assuan_ctx_from_ctrl (ctrl)), 1);
-  if (fd == -1)
+  fd = assuan_get_output_fd (get_assuan_ctx_from_ctrl (ctrl));
+  if (fd == GNUPG_INVALID_FD)
     return 0;  /* No Output command active.  */
+  else
+    {
+      out_fp = open_stream_nc (fd, "w");
+      if (!out_fp)
+        return gpg_err_code_from_syserror ();
+    }
 
-  ctrl->server_local->outstream = es_fdopen_nc (fd, "w");
-  if (!ctrl->server_local->outstream)
-    return gpg_err_code_from_syserror ();
+  ctrl->server_local->outstream = out_fp;
   return 0;
 }
 
@@ -173,6 +177,23 @@ kbxd_writen (estream_t fp, const void *buffer, size_t length)
   return err;
 }
 
+/* This status functions expects a printf style format string.  */
+gpg_error_t
+kbxd_status_printf (ctrl_t ctrl, const char *keyword, const char *format, ...)
+{
+  gpg_error_t err;
+  va_list arg_ptr;
+  assuan_context_t ctx = get_assuan_ctx_from_ctrl (ctrl);
+
+  if (!ctx) /* Oops - no assuan context.  */
+    return gpg_error (GPG_ERR_NOT_PROCESSED);
+
+  va_start (arg_ptr, format);
+  err = vprint_assuan_status (ctx, keyword, format, arg_ptr);
+  va_end (arg_ptr);
+  return err;
+}
+
 
 /* A wrapper around assuan_send_data which makes debugging the output
  * in verbose mode easier.  It also takes CTRL as argument.  */
@@ -975,10 +996,6 @@ kbxd_start_command_handler (ctrl_t ctrl, gnupg_fd_t fd, unsigned int session_id)
   ctrl->server_local->next_session = session_list;
   session_list = ctrl->server_local;
 
-
-  /* The next call enable the use of status_printf.  */
-  set_assuan_context_func (get_assuan_ctx_from_ctrl);
-
   for (;;)
     {
       rc = assuan_accept (ctx);
@@ -1029,7 +1046,6 @@ kbxd_start_command_handler (ctrl_t ctrl, gnupg_fd_t fd, unsigned int session_id)
 
   assuan_close_output_fd (ctx);
 
-  set_assuan_context_func (NULL);
   ctrl->server_local->assuan_ctx = NULL;
   assuan_release (ctx);
 
