@@ -24,7 +24,6 @@
 #include <string.h>
 #include <errno.h>
 #include <time.h>
-#include <assert.h>
 
 #include "gpgsm.h"
 #include <gcrypt.h>
@@ -47,7 +46,7 @@ struct duptable_s
   struct duptable_s *next;
 
   /* Note that we only need to store 19 bytes because the first byte
-     is implictly given by the table index (we require at least 8
+     is implicitly given by the table index (we require at least 8
      bits). */
   unsigned char fpr[19];
 };
@@ -150,7 +149,7 @@ gpgsm_export (ctrl_t ctrl, strlist_t names, estream_t stream)
       goto leave;
     }
 
-  hd = keydb_new ();
+  hd = keydb_new (ctrl);
   if (!hd)
     {
       log_error ("keydb_new failed\n");
@@ -198,8 +197,6 @@ gpgsm_export (ctrl_t ctrl, strlist_t names, estream_t stream)
     {
       for (i=0; (i < ndesc
                  && (desc[i].mode == KEYDB_SEARCH_MODE_FPR
-                     || desc[i].mode == KEYDB_SEARCH_MODE_FPR20
-                     || desc[i].mode == KEYDB_SEARCH_MODE_FPR16
                      || desc[i].mode == KEYDB_SEARCH_MODE_KEYGRIP)); i++)
         ;
       if (i == ndesc)
@@ -298,7 +295,7 @@ gpgsm_export (ctrl_t ctrl, strlist_t names, estream_t stream)
       ksba_cert_release (cert);
       cert = NULL;
     }
-  if (rc && rc != -1)
+  if (rc && gpg_err_code (rc) != GPG_ERR_NOT_FOUND)
     log_error ("keydb_search failed: %s\n", gpg_strerror (rc));
   else if (b64writer)
     {
@@ -321,7 +318,7 @@ gpgsm_export (ctrl_t ctrl, strlist_t names, estream_t stream)
 
 /* Export a certificate and its private key.  RAWMODE controls the
    actual output:
-       0 - Private key and certifciate in PKCS#12 format
+       0 - Private key and certificate in PKCS#12 format
        1 - Only unencrypted private key in PKCS#8 format
        2 - Only unencrypted private key in PKCS#1 format
     */
@@ -341,7 +338,7 @@ gpgsm_p12_export (ctrl_t ctrl, const char *name, estream_t stream, int rawmode)
   void *data;
   size_t datalen;
 
-  hd = keydb_new ();
+  hd = keydb_new (ctrl);
   if (!hd)
     {
       log_error ("keydb_new failed\n");
@@ -392,7 +389,7 @@ gpgsm_p12_export (ctrl_t ctrl, const char *name, estream_t stream, int rawmode)
             }
           err = gpg_error (GPG_ERR_AMBIGUOUS_NAME);
         }
-      else if (err == -1 || gpg_err_code (err) == GPG_ERR_EOF)
+      else if (gpg_err_code (err) == GPG_ERR_NOT_FOUND)
         err = 0;
       if (err)
         {
@@ -430,8 +427,11 @@ gpgsm_p12_export (ctrl_t ctrl, const char *name, estream_t stream, int rawmode)
                   opt.p12_charset);
     }
 
+
   if (rawmode == 0)
     ctrl->pem_name = "PKCS12";
+  else if (gpgsm_get_key_algo_info (cert, NULL, NULL) == GCRY_PK_ECC)
+    ctrl->pem_name = "EC PRIVATE KEY";
   else if (rawmode == 1)
     ctrl->pem_name = "PRIVATE KEY";
   else
@@ -479,12 +479,13 @@ gpgsm_p12_export (ctrl_t ctrl, const char *name, estream_t stream, int rawmode)
  leave:
   gnupg_ksba_destroy_writer (b64writer);
   ksba_cert_release (cert);
+  xfree (keygrip);
   xfree (desc);
   keydb_release (hd);
 }
 
 
-/* Print some info about the certifciate CERT to FP or STREAM */
+/* Print some info about the certificate CERT to FP or STREAM */
 static void
 print_short_info (ksba_cert_t cert, estream_t stream)
 {
@@ -552,7 +553,6 @@ sexp_to_kparms (gcry_sexp_t sexp)
   const char *s;
   size_t n;
   int idx;
-  const char *elems;
   gcry_mpi_t *array;
 
   list = gcry_sexp_find_token (sexp, "private-key", 0 );
@@ -562,49 +562,67 @@ sexp_to_kparms (gcry_sexp_t sexp)
   gcry_sexp_release (list);
   list = l2;
   name = gcry_sexp_nth_data (list, 0, &n);
-  if(!name || n != 3 || memcmp (name, "rsa", 3))
+  if (name && n == 3 && !memcmp (name, "rsa", 3))
     {
-      gcry_sexp_release (list);
-      return NULL;
-    }
+      /* Parameter names used with RSA in the pkcs#12 order. */
+      const char *elems = "nedqp--u";
 
-  /* Parameter names used with RSA in the pkcs#12 order. */
-  elems = "nedqp--u";
-  array = xtrycalloc (strlen(elems) + 1, sizeof *array);
-  if (!array)
-    {
-      gcry_sexp_release (list);
-      return NULL;
-    }
-  for (idx=0, s=elems; *s; s++, idx++ )
-    {
-      if (*s == '-')
-        continue; /* Computed below  */
-      l2 = gcry_sexp_find_token (list, s, 1);
-      if (l2)
+      array = xtrycalloc (strlen(elems) + 1, sizeof *array);
+      if (!array)
         {
-          array[idx] = gcry_sexp_nth_mpi (l2, 1, GCRYMPI_FMT_USG);
-          gcry_sexp_release (l2);
-        }
-      if (!array[idx]) /* Required parameter not found or invalid.  */
-        {
-          for (idx=0; array[idx]; idx++)
-            gcry_mpi_release (array[idx]);
-          xfree (array);
           gcry_sexp_release (list);
           return NULL;
         }
+      for (idx=0, s=elems; *s; s++, idx++ )
+        {
+          if (*s == '-')
+            continue; /* Computed below  */
+          l2 = gcry_sexp_find_token (list, s, 1);
+          if (l2)
+            {
+              array[idx] = gcry_sexp_nth_mpi (l2, 1, GCRYMPI_FMT_USG);
+              gcry_sexp_release (l2);
+            }
+          if (!array[idx]) /* Required parameter not found or invalid.  */
+            {
+              for (idx=0; array[idx]; idx++)
+                gcry_mpi_release (array[idx]);
+              xfree (array);
+              gcry_sexp_release (list);
+              return NULL;
+            }
+        }
+
+      array[5] = gcry_mpi_snew (0);  /* compute d mod (q-1) */
+      gcry_mpi_sub_ui (array[5], array[3], 1);
+      gcry_mpi_mod (array[5], array[2], array[5]);
+
+      array[6] = gcry_mpi_snew (0);  /* compute d mod (p-1) */
+      gcry_mpi_sub_ui (array[6], array[4], 1);
+      gcry_mpi_mod (array[6], array[2], array[6]);
     }
+  else if (name && n == 3 && !memcmp (name, "ecc", 3))
+    {
+      array = xtrycalloc (3 + 1, sizeof *array);
+      if (!array)
+        {
+          gcry_sexp_release (list);
+          return NULL;
+        }
+
+      if (gcry_sexp_extract_param (list, NULL,  "/'curve'qd",
+                                   array+0, array+1, array+2, NULL))
+        {
+          xfree (array);
+          array = NULL;  /* Error.  */
+        }
+    }
+  else
+    {
+      array = NULL;
+    }
+
   gcry_sexp_release (list);
-
-  array[5] = gcry_mpi_snew (0);  /* compute d mod (q-1) */
-  gcry_mpi_sub_ui (array[5], array[3], 1);
-  gcry_mpi_mod (array[5], array[2], array[5]);
-
-  array[6] = gcry_mpi_snew (0);  /* compute d mod (p-1) */
-  gcry_mpi_sub_ui (array[6], array[4], 1);
-  gcry_mpi_mod (array[6], array[3], array[6]);
-
   return array;
 }
 
@@ -706,8 +724,8 @@ export_p12 (ctrl_t ctrl, const unsigned char *certimg, size_t certimglen,
     {
       err = gpgsm_agent_ask_passphrase
         (ctrl,
-         i18n_utf8 ("Please enter the passphrase to protect the "
-                    "new PKCS#12 object."),
+         i18n_utf8 (N_("Please enter the passphrase to protect the "
+                       "new PKCS#12 object.")),
          1, &passphrase);
       if (err)
         goto leave;
