@@ -15,6 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
 /* GnuPG comes with a shell script gpg-zip which creates archive files
@@ -27,19 +28,21 @@
    gpg.  So here we go.  */
 
 #include <config.h>
+
 #include <ctype.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 
-#include "util.h"
-#include "i18n.h"
-#include "sysutils.h"
+#define INCLUDED_BY_MAIN_MODULE 1
+#include "../common/util.h"
+#include "../common/i18n.h"
+#include "../common/sysutils.h"
 #include "../common/openpgpdefs.h"
 #include "../common/init.h"
 #include "../common/strlist.h"
+#include "../common/comopt.h"
 
 #include "gpgtar.h"
 
@@ -72,18 +75,29 @@ enum cmd_and_opt_values
     oCMS,
     oSetFilename,
     oNull,
+    oUtf8Strings,
+    oNoCompress,
+
+    oBatch,
+    oAnswerYes,
+    oAnswerNo,
+    oStatusFD,
+    oRequireCompliance,
+    oWithLog,
 
     /* Compatibility with gpg-zip.  */
     oGpgArgs,
     oTarArgs,
+    oTarProgram,
 
     /* Debugging.  */
-    oDryRun,
+    oDebug,
+    oDryRun
   };
 
 
 /* The list of commands and options. */
-static ARGPARSE_OPTS opts[] = {
+static gpgrt_opt_t opts[] = {
   ARGPARSE_group (300, N_("@Commands:\n ")),
 
   ARGPARSE_c (aCreate,    "create",  N_("create an archive")),
@@ -108,24 +122,41 @@ static ARGPARSE_OPTS opts[] = {
   ARGPARSE_s_s (oSetFilename, "set-filename", "@"),
   ARGPARSE_s_n (oOpenPGP, "openpgp", "@"),
   ARGPARSE_s_n (oCMS, "cms", "@"),
+  ARGPARSE_s_n (oNoCompress,   "no-compress", "@"),
+
+  ARGPARSE_s_n (oBatch, "batch", "@"),
+  ARGPARSE_s_n (oAnswerYes, "yes", "@"),
+  ARGPARSE_s_n (oAnswerNo, "no", "@"),
+  ARGPARSE_s_s (oStatusFD, "status-fd", "@"),
+  ARGPARSE_s_n (oRequireCompliance, "require-compliance", "@"),
+  ARGPARSE_s_n (oWithLog, "with-log", "@"),
 
   ARGPARSE_group (302, N_("@\nTar options:\n ")),
 
   ARGPARSE_s_s (oDirectory, "directory",
-                N_("|DIRECTORY|extract files into DIRECTORY")),
+                N_("|DIRECTORY|change to DIRECTORY first")),
   ARGPARSE_s_s (oFilesFrom, "files-from",
                 N_("|FILE|get names to create from FILE")),
   ARGPARSE_s_n (oNull, "null", N_("-T reads null-terminated names")),
+#ifdef HAVE_W32_SYSTEM
+  ARGPARSE_s_n (oUtf8Strings, "utf8-strings",
+                N_("-T reads UTF-8 encoded names")),
+#else
+  ARGPARSE_s_n (oUtf8Strings, "utf8-strings", "@"),
+#endif
 
   ARGPARSE_s_s (oGpgArgs, "gpg-args", "@"),
   ARGPARSE_s_s (oTarArgs, "tar-args", "@"),
+  ARGPARSE_s_s (oTarProgram, "tar", "@"),
+
+  ARGPARSE_s_s (oDebug, "debug", "@"),
 
   ARGPARSE_end ()
 };
 
 
 /* The list of commands and options for tar that we understand. */
-static ARGPARSE_OPTS tar_opts[] = {
+static gpgrt_opt_t tar_opts[] = {
   ARGPARSE_s_s (oDirectory, "directory",
                 N_("|DIRECTORY|extract files into DIRECTORY")),
   ARGPARSE_s_s (oFilesFrom, "files-from",
@@ -136,6 +167,14 @@ static ARGPARSE_OPTS tar_opts[] = {
 };
 
 
+/* Global flags.  */
+static enum cmd_and_opt_values cmd = 0;
+static int skip_crypto = 0;
+static const char *files_from = NULL;
+static int null_names = 0;
+static int any_debug;
+
+
 
 /* Print usage information and provide strings for help. */
 static const char *
@@ -145,9 +184,11 @@ my_strusage( int level )
 
   switch (level)
     {
+    case  9: p = "GPL-3.0-or-later"; break;
     case 11: p = "@GPGTAR@ (@GNUPG@)";
       break;
     case 13: p = VERSION; break;
+    case 14: p = GNUPG_DEF_COPYRIGHT_LINE; break;
     case 17: p = PRINTABLE_OS_NAME; break;
     case 19: p = _("Please report bugs to <@EMAIL@>.\n"); break;
 
@@ -169,23 +210,25 @@ my_strusage( int level )
 static void
 set_cmd (enum cmd_and_opt_values *ret_cmd, enum cmd_and_opt_values new_cmd)
 {
-  enum cmd_and_opt_values cmd = *ret_cmd;
+  enum cmd_and_opt_values c = *ret_cmd;
 
-  if (!cmd || cmd == new_cmd)
-    cmd = new_cmd;
-  else if (cmd == aSign && new_cmd == aEncrypt)
-    cmd = aSignEncrypt;
-  else if (cmd == aEncrypt && new_cmd == aSign)
-    cmd = aSignEncrypt;
+  if (!c || c == new_cmd)
+    c = new_cmd;
+  else if (c == aSign && new_cmd == aEncrypt)
+    c = aSignEncrypt;
+  else if (c == aEncrypt && new_cmd == aSign)
+    c = aSignEncrypt;
   else
     {
       log_error (_("conflicting commands\n"));
       exit (2);
     }
 
-  *ret_cmd = cmd;
+  *ret_cmd = c;
 }
+
 
+
 /* Shell-like argument splitting.
 
    For compatibility with gpg-zip we accept arguments for GnuPG and
@@ -237,7 +280,7 @@ shell_parse_stringlist (const char *str, strlist_t *r_list)
           break;
 
         case doublequote:
-          assert (s > str || !"cannot be quoted at first char");
+          log_assert (s > str || !"cannot be quoted at first char");
           if (*s == doublequote && *(s - 1) != '\\')
             quoted = unquoted;
           else
@@ -245,7 +288,7 @@ shell_parse_stringlist (const char *str, strlist_t *r_list)
           break;
 
         default:
-          assert (! "reached");
+          log_assert (! "reached");
         }
     }
 
@@ -287,21 +330,16 @@ shell_parse_argv (const char *s, int *r_argc, char ***r_argv)
   gpgrt_annotate_leaked_object (*r_argv);
   return 0;
 }
+
+
 
-/* Global flags.  */
-enum cmd_and_opt_values cmd = 0;
-int skip_crypto = 0;
-const char *files_from = NULL;
-int null_names = 0;
-
-
 /* Command line parsing.  */
 static void
-parse_arguments (ARGPARSE_ARGS *pargs, ARGPARSE_OPTS *popts)
+parse_arguments (gpgrt_argparse_t *pargs, gpgrt_opt_t *popts)
 {
   int no_more_options = 0;
 
-  while (!no_more_options && optfile_parse (NULL, NULL, NULL, pargs, popts))
+  while (!no_more_options && gpgrt_argparse (NULL, pargs, popts))
     {
       switch (pargs->r_opt)
         {
@@ -313,6 +351,8 @@ parse_arguments (ARGPARSE_ARGS *pargs, ARGPARSE_OPTS *popts)
         case oNoVerbose: opt.verbose = 0; break;
         case oFilesFrom: files_from = pargs->r.ret_str; break;
         case oNull: null_names = 1; break;
+        case oUtf8Strings: opt.utf8strings = 1; break;
+        case oNoCompress:  opt.no_compress = 1; break;
 
 	case aList:
         case aDecrypt:
@@ -355,6 +395,13 @@ parse_arguments (ARGPARSE_ARGS *pargs, ARGPARSE_OPTS *popts)
         case oOpenPGP: /* Dummy option for now.  */ break;
         case oCMS:     /* Dummy option for now.  */ break;
 
+        case oBatch: opt.batch = 1; break;
+        case oAnswerYes: opt.answer_yes = 1; break;
+        case oAnswerNo: opt.answer_no = 1; break;
+        case oStatusFD: opt.status_fd = pargs->r.ret_str; break;
+        case oRequireCompliance: opt.require_compliance = 1; break;
+        case oWithLog: opt.with_log = 1; break;
+
         case oGpgArgs:;
           {
             strlist_t list;
@@ -371,7 +418,10 @@ parse_arguments (ARGPARSE_ARGS *pargs, ARGPARSE_OPTS *popts)
           }
           break;
 
-        case oTarArgs:;
+        case oTarProgram:  /* Dummy option.  */
+          break;
+
+        case oTarArgs:
           {
             int tar_argc;
             char **tar_argv;
@@ -381,11 +431,12 @@ parse_arguments (ARGPARSE_ARGS *pargs, ARGPARSE_OPTS *popts)
                          pargs->r.ret_str);
             else
               {
-                ARGPARSE_ARGS tar_args;
+                gpgrt_argparse_t tar_args;
                 tar_args.argc = &tar_argc;
                 tar_args.argv = &tar_argv;
                 tar_args.flags = ARGPARSE_FLAG_ARG0;
                 parse_arguments (&tar_args, tar_opts);
+                gpgrt_argparse (NULL, &tar_args, NULL);
                 if (tar_args.err)
                   log_error ("unsupported tar arguments '%s'\n",
                              pargs->r.ret_str);
@@ -394,6 +445,10 @@ parse_arguments (ARGPARSE_ARGS *pargs, ARGPARSE_OPTS *popts)
           }
           break;
 
+        case oDebug:
+          any_debug = 1;
+          break;
+
         case oDryRun:
           opt.dry_run = 1;
           break;
@@ -403,6 +458,7 @@ parse_arguments (ARGPARSE_ARGS *pargs, ARGPARSE_OPTS *popts)
     }
 }
 
+
 
 /* gpgtar main. */
 int
@@ -410,32 +466,40 @@ main (int argc, char **argv)
 {
   gpg_error_t err;
   const char *fname;
-  ARGPARSE_ARGS pargs;
-
-  assert (sizeof (struct ustar_raw_header) == 512);
+  gpgrt_argparse_t pargs;
 
   gnupg_reopen_std (GPGTAR_NAME);
-  set_strusage (my_strusage);
-  log_set_prefix (GPGTAR_NAME, GPGRT_LOG_WITH_PREFIX);
+  gpgrt_set_strusage (my_strusage);
+  log_set_prefix (GPGTAR_NAME, GPGRT_LOG_WITH_PREFIX|GPGRT_LOG_NO_REGISTRY);
 
   /* Make sure that our subsystems are ready.  */
   i18n_init();
   init_common_subsystems (&argc, &argv);
+  gnupg_init_signals (0, NULL);
+
+  log_assert (sizeof (struct ustar_raw_header) == 512);
+
+  /* Set default options */
+  opt.status_fd = NULL;
+
+  /* The configuration directories for use by gpgrt_argparser.  */
+  gpgrt_set_confdir (GPGRT_CONFDIR_SYS, gnupg_sysconfdir ());
+  gpgrt_set_confdir (GPGRT_CONFDIR_USER, gnupg_homedir ());
 
   /* Parse the command line. */
   pargs.argc  = &argc;
   pargs.argv  = &argv;
   pargs.flags = ARGPARSE_FLAG_KEEP;
   parse_arguments (&pargs, opts);
-
-  if ((files_from && !null_names) || (!files_from && null_names))
-    log_error ("--files-from and --null may only be used in conjunction\n");
-  if (files_from && strcmp (files_from, "-"))
-    log_error ("--files-from only supports argument \"-\"\n");
+  gpgrt_argparse (NULL, &pargs, NULL);
 
   if (log_get_errorcount (0))
     exit (2);
 
+  /* Get a log file from common.conf.  */
+  if (!parse_comopt (GNUPG_MODULE_NAME_GPGTAR, any_debug) && comopt.logfile)
+    log_set_file (comopt.logfile);
+
   /* Print a warning if an argument looks like an option.  */
   if (!opt.quiet && !(pargs.flags & ARGPARSE_FLAG_STOP_SEEN))
     {
@@ -446,6 +510,37 @@ main (int argc, char **argv)
           log_info (_("NOTE: '%s' is not considered an option\n"), argv[i]);
     }
 
+  /* Set status stream for our own use of --status-fd.  The original
+   * status fd is passed verbatim to gpg.  */
+  if (opt.status_fd)
+    {
+      es_syshd_t syshd;
+
+      err = gnupg_parse_fdstr (opt.status_fd, &syshd);
+      if (err)
+        log_fatal ("status-fd is invalid: %s\n", gpg_strerror (err));
+
+      if (syshd.type == ES_SYSHD_FD && syshd.u.fd == 1)
+        {
+          opt.status_stream = es_stdout;
+          if (!skip_crypto)
+            log_fatal ("using stdout for the status-fd is not possible\n");
+        }
+      else if (syshd.type == ES_SYSHD_FD && syshd.u.fd == 2)
+        opt.status_stream = es_stderr;
+      else
+        {
+          opt.status_stream = es_sysopen (&syshd, "w");
+          if (opt.status_stream)
+            es_setvbuf (opt.status_stream, NULL, _IOLBF, 0);
+        }
+      if (!opt.status_stream)
+        {
+          log_fatal ("can't open fd %s for status output: %s\n",
+                     opt.status_fd, strerror (errno));
+        }
+    }
+
   if (! opt.gpg_program)
     opt.gpg_program = gnupg_module_name (GNUPG_MODULE_NAME_GPG);
 
@@ -454,28 +549,40 @@ main (int argc, char **argv)
 
   switch (cmd)
     {
+    case aDecrypt:
     case aList:
       if (argc > 1)
-        usage (1);
-      fname = argc ? *argv : NULL;
+        gpgrt_usage (1);
+      fname = (argc && strcmp (*argv, "-"))? *argv : NULL;
       if (opt.filename)
         log_info ("note: ignoring option --set-filename\n");
       if (files_from)
         log_info ("note: ignoring option --files-from\n");
-      err = gpgtar_list (fname, !skip_crypto);
-      if (err && log_get_errorcount (0) == 0)
-        log_error ("listing archive failed: %s\n", gpg_strerror (err));
+      if (cmd == aDecrypt)
+        {
+          err = gpgtar_extract (fname, !skip_crypto);
+          if (err && !log_get_errorcount (0))
+            log_error ("extracting archive failed: %s\n", gpg_strerror (err));
+        }
+      else
+        {
+          err = gpgtar_list (fname, !skip_crypto);
+          if (err && !log_get_errorcount (0))
+            log_error ("listing archive failed: %s\n", gpg_strerror (err));
+        }
       break;
 
     case aEncrypt:
     case aSign:
     case aSignEncrypt:
-      if ((!argc && !null_names)
-          || (argc && null_names))
-        usage (1);
+      if ((!argc && !files_from)
+          || (argc && files_from))
+        gpgrt_usage (1);
       if (opt.filename)
         log_info ("note: ignoring option --set-filename\n");
-      err = gpgtar_create (null_names? NULL :argv,
+      err = gpgtar_create (files_from? NULL : argv,
+                           files_from,
+                           null_names,
                            !skip_crypto
                            && (cmd == aEncrypt || cmd == aSignEncrypt),
                            cmd == aSign || cmd == aSignEncrypt);
@@ -483,30 +590,27 @@ main (int argc, char **argv)
         log_error ("creating archive failed: %s\n", gpg_strerror (err));
       break;
 
-    case aDecrypt:
-      if (argc != 1)
-        usage (1);
-      if (opt.outfile)
-        log_info ("note: ignoring option --output\n");
-      if (files_from)
-        log_info ("note: ignoring option --files-from\n");
-      fname = argc ? *argv : NULL;
-      err = gpgtar_extract (fname, !skip_crypto);
-      if (err && log_get_errorcount (0) == 0)
-        log_error ("extracting archive failed: %s\n", gpg_strerror (err));
-      break;
-
     default:
       log_error (_("invalid command (there is no implicit command)\n"));
+      err = 0;
       break;
     }
 
+  if (opt.status_stream)
+    {
+      if (err || log_get_errorcount (0))
+        es_fprintf (opt.status_stream, "[GNUPG:] FAILURE - %u\n",
+                    err? err : gpg_error (GPG_ERR_GENERAL));
+      else
+        es_fprintf (opt.status_stream, "[GNUPG:] SUCCESS\n");
+    }
+
   return log_get_errorcount (0)? 1:0;
 }
 
 
 /* Read the next record from STREAM.  RECORD is a buffer provided by
-   the caller and must be at leadt of size RECORDSIZE.  The function
+   the caller and must be at least of size RECORDSIZE.  The function
    return 0 on success and error code on failure; a diagnostic
    printed as well.  Note that there is no need for an EOF indicator
    because a tarball has an explicit EOF record. */
