@@ -190,10 +190,7 @@ static gnupg_fd_t create_server_socket (const char *name,
                                         assuan_sock_nonce_t *nonce);
 
 static void *start_connection_thread (void *arg);
-static void handle_connections (int listen_fd);
-
-/* Pth wrapper function definitions. */
-ASSUAN_SYSTEM_NPTH_IMPL;
+static void handle_connections (gnupg_fd_t listen_fd);
 
 static int active_connections;
 
@@ -368,7 +365,6 @@ main (int argc, char **argv )
   malloc_hooks.free = gcry_free;
   assuan_set_malloc_hooks (&malloc_hooks);
   assuan_set_gpg_err_source (GPG_ERR_SOURCE_DEFAULT);
-  assuan_set_system_hooks (ASSUAN_SYSTEM_NPTH);
   assuan_sock_init ();
   setup_libassuan_logging (&opt.debug, NULL);
 
@@ -415,12 +411,12 @@ main (int argc, char **argv )
   */
 
 
-  /* The configuraton directories for use by gpgrt_argparser.  */
+  /* The configuration directories for use by gpgrt_argparser.  */
   gpgrt_set_confdir (GPGRT_CONFDIR_SYS, gnupg_sysconfdir ());
   gpgrt_set_confdir (GPGRT_CONFDIR_USER, gnupg_homedir ());
 
   /* We are re-using the struct, thus the reset flag.  We OR the
-   * flags so that the internal intialized flag won't be cleared. */
+   * flags so that the internal initialized flag won't be cleared. */
   argc = orig_argc;
   argv = orig_argv;
   pargs.argc = &argc;
@@ -584,7 +580,7 @@ main (int argc, char **argv )
       /* This is the simple pipe based server */
       ctrl_t ctrl;
       npth_attr_t tattr;
-      int fd = -1;
+      gnupg_fd_t fd = GNUPG_INVALID_FD;
 
 #ifndef HAVE_W32_SYSTEM
       {
@@ -599,6 +595,7 @@ main (int argc, char **argv )
 
       npth_init ();
       gpgrt_set_syscall_clamp (npth_unprotect, npth_protect);
+      assuan_control (ASSUAN_CONTROL_REINIT_SYSCALL_CLAMP, NULL);
 
       /* If --debug-allow-core-dump has been given we also need to
          switch the working directory to a place where we can actually
@@ -618,9 +615,9 @@ main (int argc, char **argv )
       if (multi_server)
         {
           socket_name = create_socket_name (TPM2DAEMON_SOCK_NAME);
-          fd = FD2INT (create_server_socket (socket_name,
-					     &redir_socket_name,
-					     &socket_nonce));
+          fd = create_server_socket (socket_name,
+                                     &redir_socket_name,
+                                     &socket_nonce);
         }
 
       res = npth_attr_init (&tattr);
@@ -654,8 +651,8 @@ main (int argc, char **argv )
       /* We run handle_connection to wait for the shutdown signal and
          to run the ticker stuff.  */
       handle_connections (fd);
-      if (fd != -1)
-        close (fd);
+      if (fd != GNUPG_INVALID_FD)
+        assuan_sock_close (fd);
     }
   else if (!is_daemon)
     {
@@ -664,7 +661,7 @@ main (int argc, char **argv )
     }
   else
     { /* Regular server mode */
-      int fd;
+      gnupg_fd_t fd;
 #ifndef HAVE_W32_SYSTEM
       pid_t pid;
       int i;
@@ -672,8 +669,8 @@ main (int argc, char **argv )
 
       /* Create the socket.  */
       socket_name = create_socket_name (TPM2DAEMON_SOCK_NAME);
-      fd = FD2INT (create_server_socket (socket_name,
-                                         &redir_socket_name, &socket_nonce));
+      fd = create_server_socket (socket_name,
+                                 &redir_socket_name, &socket_nonce);
 
 
       fflush (NULL);
@@ -740,6 +737,7 @@ main (int argc, char **argv )
 
       npth_init ();
       gpgrt_set_syscall_clamp (npth_unprotect, npth_protect);
+      assuan_control (ASSUAN_CONTROL_REINIT_SYSCALL_CLAMP, NULL);
 
       /* Detach from tty and put process into a new session. */
       if (!nodetach )
@@ -788,7 +786,7 @@ main (int argc, char **argv )
 
       handle_connections (fd);
 
-      close (fd);
+      assuan_sock_close (fd);
     }
 
   xfree (config_filename);
@@ -897,7 +895,7 @@ handle_signal (int signo)
 /* Create a name for the socket.  We check for valid characters as
    well as against a maximum allowed length for a unix domain socket
    is done.  The function terminates the process in case of an error.
-   Retunrs: Pointer to an allcoated string with the absolute name of
+   Returns: Pointer to an allocated string with the absolute name of
    the socket used.  */
 static char *
 create_socket_name (char *standard_name)
@@ -1012,7 +1010,7 @@ start_connection_thread (void *arg)
       && assuan_sock_check_nonce (ctrl->thread_startup.fd, &socket_nonce))
     {
       log_info (_("error reading nonce on fd %d: %s\n"),
-                FD2INT (ctrl->thread_startup.fd), strerror (errno));
+                FD_DBG (ctrl->thread_startup.fd), strerror (errno));
       assuan_sock_close (ctrl->thread_startup.fd);
       xfree (ctrl);
       return NULL;
@@ -1023,19 +1021,19 @@ start_connection_thread (void *arg)
   tpm2d_init_default_ctrl (ctrl);
   if (opt.verbose)
     log_info (_("handler for fd %d started\n"),
-              FD2INT (ctrl->thread_startup.fd));
+              FD_DBG (ctrl->thread_startup.fd));
 
   /* If this is a pipe server, we request a shutdown if the command
      handler asked for it.  With the next ticker event and given that
      no other connections are running the shutdown will then
      happen.  */
-  if (tpm2d_command_handler (ctrl, FD2INT (ctrl->thread_startup.fd))
+  if (tpm2d_command_handler (ctrl, ctrl->thread_startup.fd)
       && pipe_server)
     shutdown_pending = 1;
 
   if (opt.verbose)
     log_info (_("handler for fd %d terminated\n"),
-              FD2INT (ctrl->thread_startup.fd));
+              FD_DBG (ctrl->thread_startup.fd));
 
   tpm2d_deinit_default_ctrl (ctrl);
   xfree (ctrl);
@@ -1075,7 +1073,7 @@ tpm2d_kick_the_loop (void)
    in which case this code will only do regular timeouts and handle
    signals. */
 static void
-handle_connections (int listen_fd)
+handle_connections (gnupg_fd_t listen_fd)
 {
   npth_attr_t tattr;
   struct sockaddr_un paddr;
@@ -1083,7 +1081,6 @@ handle_connections (int listen_fd)
   fd_set fdset, read_fdset;
   int nfd;
   int ret;
-  int fd;
   struct timespec timeout;
   struct timespec *t;
   int saved_errno;
@@ -1152,10 +1149,10 @@ handle_connections (int listen_fd)
 
   FD_ZERO (&fdset);
   nfd = 0;
-  if (listen_fd != -1)
+  if (listen_fd != GNUPG_INVALID_FD)
     {
-      FD_SET (listen_fd, &fdset);
-      nfd = listen_fd;
+      FD_SET (FD2INT (listen_fd), &fdset);
+      nfd = FD2NUM (listen_fd);
     }
 
   for (;;)
@@ -1173,7 +1170,7 @@ handle_connections (int listen_fd)
              file descriptors to wait for, so that the select will be
              used to just wait on a signal or timeout event. */
           FD_ZERO (&fdset);
-          listen_fd = -1;
+          listen_fd = GNUPG_INVALID_FD;
         }
 
       periodical_check = 0;
@@ -1232,13 +1229,16 @@ handle_connections (int listen_fd)
         }
 #endif
 
-      if (listen_fd != -1 && FD_ISSET (listen_fd, &read_fdset))
+      if (listen_fd != GNUPG_INVALID_FD
+          && FD_ISSET (FD2INT (listen_fd), &read_fdset))
         {
           ctrl_t ctrl;
+          gnupg_fd_t fd;
 
           plen = sizeof paddr;
-          fd = npth_accept (listen_fd, (struct sockaddr *)&paddr, &plen);
-          if (fd == -1)
+          fd = assuan_sock_accept (listen_fd,
+                                   (struct sockaddr *)&paddr, &plen);
+          if (fd == GNUPG_INVALID_FD)
             {
               log_error ("accept failed: %s\n", strerror (errno));
             }
@@ -1246,22 +1246,22 @@ handle_connections (int listen_fd)
             {
               log_error ("error allocating connection control data: %s\n",
                          strerror (errno) );
-              close (fd);
+              assuan_sock_close (fd);
             }
           else
             {
               char threadname[50];
               npth_t thread;
 
-              snprintf (threadname, sizeof threadname, "conn fd=%d", fd);
-              ctrl->thread_startup.fd = INT2FD (fd);
+              snprintf (threadname, sizeof threadname, "conn fd=%d", FD_DBG (fd));
+              ctrl->thread_startup.fd = fd;
               ret = npth_create (&thread, &tattr, start_connection_thread, ctrl);
               if (ret)
                 {
                   log_error ("error spawning connection handler: %s\n",
                              strerror (ret));
                   xfree (ctrl);
-                  close (fd);
+                  assuan_sock_close (fd);
                 }
               else
                 npth_setname_np (thread, threadname);
